//ICC-AVR application builder : 05.01.2014 0:17:55
// Target : M8
// Crystal: 16.000Mhz

#include <iom8v.h>
#include <macros.h>

unsigned char tx_arr[8] = {0, 0, 0, 0, 0, 0, 0, 0};
unsigned char rx_arr[8] = {0, 0, 0, 0, 0, 0, 0, 0};
unsigned char rx_count = 0;
unsigned char f_rx = 0;
unsigned long N_tx = 0;
unsigned char err = 0;
float f1 = 0;
unsigned char c1 = 0;
unsigned long i1 = 0;

unsigned char temp_TCCR2 = 0;
unsigned char gen_wait = 0;

unsigned char gen_2s = 0;
unsigned char gen_2s_count = 1;
unsigned char gen_2s_state = 0;
unsigned char gen_2s_wait = 0;

#pragma interrupt_handler int1_isr:iv_INT1
void int1_isr(void)
{
 //external interupt on INT1
 TCCR2 = temp_TCCR2;// 
 if(temp_TCCR2) PORTC &= (~0x01);
 else PORTC |= 0x01;
 gen_wait = 0;
 
 if(gen_2s_wait == 1)
 {
   TCCR1B = 0x05; //start Timer
   gen_2s_wait = 0;
   PORTC &= (~0x01);
   PORTB &= (~0x08);//    0
 }
}

void port_init(void)
{
 PORTB = 0x00;
 DDRB  = 0xFF;
 PORTC = 0x03;
 DDRC  = 0x3F;
 PORTD = 0x00;
 DDRD  = 0xFC;
}

//UART0 initialize
// desired baud rate: 1000000
// actual: baud rate:1000000 (0,0%)
// char size: 8 bit
// parity: Disabled
void uart0_init(void)
{
 UCSRB = 0x00; //disable while setting baud rate
 UCSRA = 0x00;
 UCSRC = BIT(URSEL) | 0x06;
 UBRRL = 0x00; //set baud rate lo
 UBRRH = 0x00; //set baud rate hi
 UCSRB = 0xD0;
}

#pragma interrupt_handler uart0_rx_isr:12
void uart0_rx_isr(void)
{
 //uart has received a character in UDR
 rx_arr[rx_count] = UDR;
 rx_count++;
 TCCR0 = 0x00; //stop
 TCNT0 = 0x06; //reload counter value
 TCCR0 = 0x05; //start timer
}

//TIMER0 initialize - prescale:1024
// desired value: 10mSec
// actual value:  9,984mSec (0,2%)
void timer0_init(void)
{
 TCCR0 = 0x00; //stop
 TCNT0 = 0xB2; //set count
 //TCCR0 = 0x05; //start timer
}

#pragma interrupt_handler timer0_ovf_isr:10
void timer0_ovf_isr(void)
{
 TCCR0 = 0x00; //stop
 TCNT0 = 0xB2; //reload counter value
 UCSRB = 0x08;
 f_rx = 1;
}

//TIMER1 initialize - prescale:1024
// WGM: 0) Normal, TOP=0xFFFF
// desired value: 1Hz
// actual value:  1,000Hz (0,0%)
void timer1_init(void)
{
 TCCR1B = 0x00; //stop
 TCNT1H = 0xC2; //setup
 TCNT1L = 0xF7;
 OCR1AH = 0x3D;
 OCR1AL = 0x09;
 OCR1BH = 0x3D;
 OCR1BL = 0x09;
 ICR1H  = 0x3D;
 ICR1L  = 0x09;
 TCCR1A = 0x00;
 //TCCR1B = 0x05; //start Timer
}

#pragma interrupt_handler timer1_ovf_isr:iv_TIM1_OVF
void timer1_ovf_isr(void)
{
 //TIMER1 has overflowed
 TCNT1H = 0xC2; //reload counter high value
 TCNT1L = 0xF7; //reload counter low value
 
 if(gen_2s_state == 0)
 {
   if(gen_2s_count == (gen_2s / 2))
   {
     gen_2s_state = 1;
	 PORTB |= 0x08;
   }
 }
 else
 {
   if(gen_2s_count == gen_2s)
   {
     gen_2s_state = 0;
	 PORTB &= (~0x08);
	 gen_2s_count = 0;
   }
 }
 gen_2s_count++;
 
}

void tx_data(void)
{
 for(c1 = 0; c1 < N_tx; c1++)
 {
  UDR = tx_arr[c1];
  while((UCSRA&0x20) == 0);
 }
 UCSRB = 0x90;
}

//call this routine to initialize all peripherals
void init_devices(void)
{
 //stop errant interrupts until set up
 CLI(); //disable all interrupts
 port_init();
 timer0_init();
 timer1_init();
 uart0_init();

 MCUCR = 0x0C;
 GICR  = 0x80;
 TIMSK = 0x11|0x04; //timer interrupt sources
 SEI(); //re-enable interrupts
 //all peripherals are now initialized
}

void main(void)
{
 init_devices();
 //insert your functional code here...
 while(1)
 {
   
   if(gen_wait == 0)
   {
     TCCR2 = temp_TCCR2;// 
	 if(temp_TCCR2 || gen_2s && (gen_2s_wait == 0)) PORTC &= (~0x01);
	 else PORTC |= 0x01;
   }
   
   if(f_rx)// 
   {
     if(rx_arr[0] != 0x45) err |= 0x01;
     if((rx_arr[rx_count - 2] != 0x65)||(rx_arr[rx_count - 1] != 0x85)) err |= 0x02;
     if((rx_arr[1] != 0x00)&&(rx_arr[1] != 0x10)&&(rx_arr[1] != 0x11)
	 &&(rx_arr[1] != 0x1B)&&(rx_arr[1] != 0x1C)
     &&(rx_arr[1] != 0x1D)&&(rx_arr[1] != 0x1E)&&(rx_arr[1] != 0x20)
	 &&(rx_arr[1] != 0x21)&&(rx_arr[1] != 0x22)) err |= 0x04;
	 
	 rx_count = 0;
	 
	 if(!err)
     {
	   if(rx_arr[1] == 0x00)// 
	   {
	     N_tx = 4;
	   }
	   
	   if(rx_arr[1] == 0x10)//  IO
	   {
	     N_tx = 4;
         if(rx_arr[2]&0x01) PORTD |= 0x04;//IO1
	     else PORTD &= (~0x04);
	     if(rx_arr[2]&0x02) PORTB |= 0x01;//IO2
         else PORTB &= (~0x01);
	     if(rx_arr[2]&0x04) PORTB |= 0x04;//IO3
	     else PORTB &= (~0x04);
	     if(rx_arr[2]&0x08) PORTB |= 0x08;//IO4
	     else PORTB &= (~0x08);
	     if(rx_arr[2]&0x10) PORTB |= 0x10;//IO5
	     else PORTB &= (~0x10);
	     if(rx_arr[2]&0x20) PORTB |= 0x20;//IO6
	     else PORTB &= (~0x20);
	     if(rx_arr[2]&0x40) PORTC |= 0x10;//IO7
	     else PORTC &= (~0x10);
	     if(rx_arr[2]&0x80) PORTC |= 0x20;//IO8
	     else PORTC &= (~0x20);
	   }
	   
	   if(rx_arr[1] == 0x11)//  IO,   
	   {
	     N_tx = 4;
         //if(rx_arr[2]&0x01) PORTD |= 0x08;//IO1
	     //else PORTD &= (~0x08);
	     if(rx_arr[2]&0x02) PORTD |= 0x10;//IO2
         else PORTD &= (~0x10);
	     if(rx_arr[2]&0x04) PORTD |= 0x20;//IO3
	     else PORTD &= (~0x20);
	     if(rx_arr[2]&0x08) PORTD |= 0x40;//IO4
	     else PORTD &= (~0x40);
	     if(rx_arr[2]&0x10) PORTD |= 0x80;//IO5
	     else PORTD &= (~0x80);
	     if(rx_arr[2]&0x20) PORTB |= 0x02;//IO6
	     else PORTB &= (~0x02);
	     if(rx_arr[2]&0x40) PORTC |= 0x04;//IO7
	     else PORTC &= (~0x04);
	     if(rx_arr[2]&0x80)
		 {
		   PORTC |= 0x08;//IO8
		   PORTC &= (~0x02);
		 }
	     else
		 {
		   PORTC &= (~0x08);
		   PORTC |= 0x02;
		 }
	   }
	   
	   if((rx_arr[1] == 0x1D)||(rx_arr[1] == 0x1E))//   
	   {
	    if(rx_arr[1] == 0x1E)
		{
		  gen_wait = 1;
		}
		N_tx = 8;
	    i1 = ((unsigned long)(rx_arr[2])) << 24;
	    i1 |= ((unsigned long)(rx_arr[3])) << 16;
		i1 |= ((unsigned long)(rx_arr[4])) << 8;
		i1 |= ((unsigned long)(rx_arr[5]));
	    if(i1 == 0)
	    {
	     TCCR2 = 0x00; //stop
		 temp_TCCR2 = 0x00;
		 gen_wait = 0;
	    }
	    if((i1 != 0)&&(i1 < 123))// 1024
	    {
	     f1 = 7812.5/((float)(i1)) - 1;
	     if(f1 > 254) f1 = 254;
	     OCR2 = ((unsigned int)(f1));//  
	     temp_TCCR2 = 0x1F;// 
	     i1 = ((unsigned long)(7812.5 / ((unsigned char)(f1 + 1))));
	    }
	    if((i1 > 122)&&(i1 < 245))// 256
	    {
	     f1 = 31250/((float)(i1)) - 1;
	     if(f1 > 254) f1 = 254;
	     OCR2 = ((unsigned int)(f1));//  
	     temp_TCCR2 = 0x1E;// 
	     i1 = ((unsigned long)(31250 / ((unsigned char)(f1 + 1))));
	    }
	    if((i1 > 244)&&(i1 < 489))// 128
	    {
	     f1 = 62500/((float)(i1)) - 1;
	     if(f1 > 254) f1 = 254;
	     OCR2 = ((unsigned int)(f1));//  
	     temp_TCCR2 = 0x1D;// 
	     i1 = ((unsigned long)(62500 / ((unsigned char)(f1 + 1))));
	    }
	    if((i1 > 488)&&(i1 < 977))// 64
	    {
	     f1 = 125000/((float)(i1)) - 1;
	     if(f1 > 254) f1 = 254;
	     OCR2 = ((unsigned int)(f1));//  
	     temp_TCCR2 = 0x1C;// 
	     i1 = ((unsigned long)(125000 / ((unsigned char)(f1 + 1))));
	    }
	    if((i1 > 976)&&(i1 < 3907))// 32
	    {
	     f1 = 250000/((float)(i1)) - 1;
	     if(f1 > 254) f1 = 254;
	     OCR2 = ((unsigned int)(f1));//  
	     temp_TCCR2 = 0x1B;// 
	     i1 = ((unsigned long)(250000 / ((unsigned char)(f1 + 1))));
	    }
	    if((i1 > 3906)&&(i1 < 31251))// 8
	    {
	     f1 = 1000000/((float)(i1)) - 1;
	     if(f1 > 254) f1 = 254;
	     OCR2 = ((unsigned int)(f1));//  
	     temp_TCCR2 = 0x1A;// 
	     i1 = ((unsigned long)(1000000 / ((unsigned char)(f1 + 1))));
	    }
	    if(i1 > 31250)// 1
	    {
	     f1 = 8000000/((float)(i1)) - 1;
	     if(f1 > 254) f1 = 254;
	     OCR2 = ((unsigned int)(f1));//  
	     temp_TCCR2 = 0x19;// 
	     i1 = ((unsigned long)(8000000 / ((unsigned char)(f1 + 1))));
	    }
	    tx_arr[2] = (unsigned char)(i1 >> 24);
	    tx_arr[3] = ((unsigned char)((i1&0x00FF0000) >> 16));
		tx_arr[4] = ((unsigned char)((i1&0x0000FF00) >> 8));
	    tx_arr[5] = ((unsigned char)(i1&0x000000FF));
	   }
	   
	   if(rx_arr[1] == 0x1B)//    0,5 
	   {
	     N_tx = 4;
		 gen_2s = rx_arr[2];
		 TCCR1B = 0x05; //start Timer
		 
		 if(gen_2s)
		 {
		   PORTC &= (~0x01);
		 }
	     else
		 {
		   PORTC |= 0x01;
		   gen_2s_count = 1;
		   gen_2s_state = 0;
		   PORTB &= (~0x08);
		   
		   TCCR1B = 0x00; //stop
		   TCNT1H = 0xC2; //reload counter high value
		   TCNT1L = 0xF7; //reload counter low value
		 }
	   }
	   
	   if(rx_arr[1] == 0x1C)//    0,5 
	   {
	     N_tx = 4;
		 gen_2s = rx_arr[2];
		 
		 gen_2s_wait = 1;
		 //TCCR1B = 0x05; //start Timer
		 
		 if(gen_2s)
		 {
		   //PORTC &= (~0x01);
		 }
	     else
		 {
		   gen_2s_wait = 0;
		   
		   PORTC |= 0x01;
		   gen_2s_count = 1;
		   gen_2s_state = 0;
		   PORTB &= (~0x08);
		   
		   TCCR1B = 0x00; //stop
		   TCNT1H = 0xC2; //reload counter high value
		   TCNT1L = 0xF7; //reload counter low value
		 }
	   }
	   
	   tx_arr[0] = 0x55;
	   tx_arr[1] = (rx_arr[1]|0x40);
	   
	   tx_arr[N_tx - 2] = 0x75;
	   tx_arr[N_tx - 1] = 0x95;
	 }
	 else//  
     {
	   N_tx = 5;
	   tx_arr[0] = 0x35;
	   tx_arr[1] = (rx_arr[1]|0x80);
	   tx_arr[2] = err;
	   tx_arr[3] = 0x75;
	   tx_arr[4] = 0x95;
     }
     if(!((rx_arr[1] == 0x23)&&(!err)))//(    )
     {
	   tx_data();// 
     }
     for(c1 = 0; c1 < 8; c1++) rx_arr[c1] = 0;//  
     f_rx = 0;//  
   }
 }
}